/**************************************************
 Jeu Casse-brique
 
 J. Labbé - Lycée Boissy d'Anglas, Annonay (07)
 **************************************************/

/*
  Variables globales
 */

// paramètres du jeu
boolean marche;
int hauteur_bandeau = 20;
int temps_pause = 0; // en ms
int correction_temps = 0; // en ms

// paramètres de la balle
float deplacementX, deplacementY;
float x, y, r;

// paramètres de la barre (raquette)
float xbarre, ybarre, epaisseur, longueur, vitessex, vitessey;

// paramètres du mur
int nbriques=8;
Brique[] mur = new Brique[nbriques];

// ==================================================
void setup()
{
  // Paramètres de processing
  smooth();
  size(500, 500);
  
  // affectation des variables
  marche = true;

  epaisseur = height/100;
  longueur = width/5;
  xbarre = width/2;
  ybarre=height-epaisseur/2;

  r = height/30;

  int largeur_brique = width/11;
  int hauteur_brique = height/20;

  nettoyer();
  creer_mur(largeur_brique, hauteur_brique);
  
  lancer_balle(width/2);
}

// ==================================================
void draw()
/*
  fonction de Processing appelée à chaque cycle
 */
{
  if (marche) {
    nettoyer();
    bouger();
    dessiner();
  }
}

// ==================================================
void mousePressed()
/*
  fonction de Processing appelée lorsqu'un boutton de la souris est pressé
 */
{
  if (!marche) {
    marche=true;
    correction_temps = correction_temps + millis() - temps_pause;
    lancer_balle(mouseX);
  }
}

// ==================================================
void pause()
/*
  met le jeu en pause
 */
{
  if (marche) {
    marche=false;
    temps_pause = millis();
  }
}

// ==================================================
void lancer_balle(int xpos)
/* 
 lance la balle avec une vitesse aléatoire
 */
{
  x = xpos;
  y = ybarre + epaisseur/2;

  float vitesse = random(height*0.015, height*.02);
  float angle = random(-60, 60);

  deplacementX = vitesse*sin(radians(angle));
  deplacementY = -vitesse*cos(radians(angle));
}

// ==================================================
void nettoyer()
/*
 DESCRIPTION À FAIRE
 */
{  
  background(0);
}

// ==================================================
void bouger()
/*
 DESCRIPTION À FAIRE
 */
{
  x = x + deplacementX;
  y = y + deplacementY;

  vitessex = mouseX - xbarre;
  xbarre = (mouseX);

  int res = rebond();
  if (res == 1)
    deplacementY = -deplacementY;
  else if (res == 2)
    deplacementX = -deplacementX;
}

// ==================================================
void dessiner()
/*
 DESCRIPTION À FAIRE
 */
{
  // raquette
  fill(255);
  rectMode(CENTER);
  rect(xbarre, ybarre, longueur, epaisseur);

  // briques
  for (int i=0; i<nbriques; i++)
    mur[i].dessiner();

  // balle
  fill(255);
  ellipse(x, y, r, r);

  // décors  
  fill(255);
  stroke(255);
  int epaisseur_ligne=5;
  rectMode(CORNER);
  rect(0, hauteur_bandeau, epaisseur_ligne, height);
  rect(0, hauteur_bandeau, width, epaisseur_ligne);
  rect(width - epaisseur_ligne, hauteur_bandeau, epaisseur_ligne, height);

  // bandeau
  dessiner_bandeau();
}

// ==================================================
void dessiner_bandeau()
/*
 Dessine le bandeau supérieur du jeu
 */
{
  int taille_texte = 14; 

  textSize(taille_texte);
  fill(200);
  textAlign(LEFT);
  int temps = (millis() - correction_temps)/1000; // en secondes
  text("Temps : "+temps, 5, 0.5*(taille_texte+hauteur_bandeau));
}

// ==================================================
int rebond() 
/*
Détermine si la balle doit rebondir.
 Renvoie :
 0 - pas de rebond
 1 - rebond vertical
 2 - rebond latéral
 */
{

  if (x < r || x > width - r)
    return 2;

  if (y < hauteur_bandeau + r)
    return 1;

  for (int i=0; i<nbriques; i++)
  {
    int res = mur[i].est_touchee(x, y, r);
    if (res != 0) { // une des brique est touchée
      return res;
    }
  }

  if ( (x > xbarre-longueur/2-r && x < xbarre+longueur/2+r)
    && (y + r > height - epaisseur/2 && deplacementY>0))
  {
    deplacementX=deplacementX + 0.5*vitessex; // modifie la vitesse latérale si la raquette bouge
    return 1;
  }

  return 0;
}

// ==================================================
void creer_mur(int largeur, int hauteur)
/*
  création du mur de briques
 */
{
  mur[0] = new Brique();
  mur[0].ChoisirPosition(  width/5, int(height*0.15) );
  mur[0].ChoisirTaille(largeur, hauteur);
  mur[0].ChoisirEtat(6);

  mur[1] = new Brique();
  mur[1].ChoisirPosition(  width/5*2, int(height*0.15) );
  mur[1].ChoisirTaille(largeur, hauteur);
  mur[1].ChoisirEtat(6);

  mur[2] = new Brique();
  mur[2].ChoisirPosition(  width/5*3, int(height*0.15) );
  mur[2].ChoisirTaille(largeur, hauteur);
  mur[2].ChoisirEtat(6);

  mur[3] = new Brique();
  mur[3].ChoisirPosition(  width/5*4, int(height*0.15) );
  mur[3].ChoisirTaille(largeur, hauteur);
  mur[3].ChoisirEtat(6);

  mur[4] = new Brique();
  mur[4].ChoisirPosition(  width/5, int(height*0.4) );
  mur[4].ChoisirTaille(largeur, hauteur);
  mur[4].ChoisirEtat(6);

  mur[5] = new Brique();
  mur[5].ChoisirPosition(  width/5*2, int(height*0.4) );
  mur[5].ChoisirTaille(largeur, hauteur);
  mur[5].ChoisirEtat(6);

  mur[6] = new Brique();
  mur[6].ChoisirPosition(  width/5*3, int(height*0.4) );
  mur[6].ChoisirTaille(largeur, hauteur);
  mur[6].ChoisirEtat(6);

  mur[7] = new Brique();
  mur[7].ChoisirPosition(  width/5*4, int(height*0.4) );
  mur[7].ChoisirTaille(largeur, hauteur);
  mur[7].ChoisirEtat(6);

}

// ==================================================
// EN TÊTE À CRÉER
/*
 Affiche le message et arrête le jeu
 */
 /*{
  pause();

  fill(50);
  rect(0, height*0.5, width, height*0.4);
  fill(150);
  textAlign(CENTER);
  textSize(20);
  text(message, width/2, height*0.6);
  textSize(16);
  text("Cliquer pour continuer", width/2, height*0.8);

  redraw();
}*/
